@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1999 Erez Zadok
+ * Copyright (c) 1997-2000 Erez Zadok
  * Copyright (c) 1990 Jan-Simon Pendry
  * Copyright (c) 1990 Imperial College of Science, Technology & Medicine
  * Copyright (c) 1990 The Regents of the University of California.
@@ -38,7 +38,7 @@
  *
  *      %W% (Berkeley) %G%
  *
- * $Id: amq.c,v 1.3 1999/01/10 21:53:58 ezk Exp $
+ * $Id: amq.c,v 1.7.2.2 2000/04/01 01:22:13 ezk Exp $
  *
  */
 
@@ -48,13 +48,13 @@
 
 #ifndef lint
 char copyright[] = "\
-@(#)Copyright (c) 1997-1999 Erez Zadok\n\
+@(#)Copyright (c) 1997-2000 Erez Zadok\n\
 @(#)Copyright (c) 1990 Jan-Simon Pendry\n\
 @(#)Copyright (c) 1990 Imperial College of Science, Technology & Medicine\n\
 @(#)Copyright (c) 1990 The Regents of the University of California.\n\
 @(#)All rights reserved.\n";
 #if __GNUC__ < 2
-static char rcsid[] = "$Id: amq.c,v 1.3 1999/01/10 21:53:58 ezk Exp $";
+static char rcsid[] = "$Id: amq.c,v 1.7.2.2 2000/04/01 01:22:13 ezk Exp $";
 static char sccsid[] = "%W% (Berkeley) %G%";
 #endif /* __GNUC__ < 2 */
 #endif /* not lint */
@@ -74,9 +74,12 @@ static int stats_flag;
 static int getvers_flag;
 static int amd_program_number = AMQ_PROGRAM;
 static int use_tcp_flag, use_udp_flag;
+static int getpwd_flag;
 static char *debug_opts;
 static char *amq_logfile;
+#ifdef ENABLE_AMQ_MOUNT
 static char *mount_map;
+#endif /* ENABLE_AMQ_MOUNT */
 static char *xlog_optstr;
 static char localhost[] = "localhost";
 static char *def_server = localhost;
@@ -184,6 +187,31 @@ show_mti(amq_mount_tree *mt, enum show_opt e, int *mwid, int *dwid, int *twid)
   }
 }
 
+
+/*
+ * Display a pwd data
+ */
+static void
+show_pwd(amq_mount_tree *mt, char *path, int *flag)
+{
+  int len;
+
+  while (mt) {
+    len = strlen(mt->mt_mountpoint);
+    if (NSTREQ(path, mt->mt_mountpoint, len) &&
+	!STREQ(mt->mt_directory, mt->mt_mountpoint)) {
+      char buf[MAXPATHLEN+1];
+      strcpy(buf, mt->mt_directory);
+      strcat(buf, &path[len]);
+      strcpy(path, buf);
+      *flag = 1;
+    }
+    show_pwd(mt->mt_next, path, flag);
+    mt = mt->mt_child;
+  }
+}
+
+
 /*
  * Display a mount tree.
  */
@@ -197,6 +225,7 @@ show_mt(amq_mount_tree *mt, enum show_opt e, int *mwid, int *dwid, int *pwid)
   }
 }
 
+
 static void
 show_mi(amq_mount_info_list *ml, enum show_opt e, int *mwid, int *dwid, int *twid)
 {
@@ -235,7 +264,11 @@ show_mi(amq_mount_info_list *ml, enum show_opt e, int *mwid, int *dwid, int *twi
 	if (mi->mi_error > 0) {
 	  extern int sys_nerr;
 	  if (mi->mi_error < sys_nerr)
+#ifdef HAVE_STRERROR
+	    printf(" (%s)", strerror(mi->mi_error));
+#else /* not HAVE_STRERROR */
 	    printf(" (%s)", sys_errlist[mi->mi_error]);
+#endif /* not HAVE_STRERROR */
 	  else
 	    printf(" (Error %d)", mi->mi_error);
 	} else if (mi->mi_error < 0) {
@@ -325,8 +358,16 @@ main(int argc, char *argv[])
   /*
    * Parse arguments
    */
-  while ((opt_ch = getopt(argc, argv, "fh:l:msuvx:D:M:pP:TU")) != -1)
+#ifdef ENABLE_AMQ_MOUNT
+  while ((opt_ch = getopt(argc, argv, "Hfh:l:msuvx:D:M:pP:TUw")) != -1)
+#else /* not ENABLE_AMQ_MOUNT */
+  while ((opt_ch = getopt(argc, argv, "Hfh:l:msuvx:D:pP:TUw")) != -1)
+#endif /* not ENABLE_AMQ_MOUNT */
     switch (opt_ch) {
+    case 'H':
+      goto show_usage;
+      break;
+
     case 'f':
       flush_flag = 1;
       nodefault = 1;
@@ -376,10 +417,12 @@ main(int argc, char *argv[])
       nodefault = 1;
       break;
 
+#ifdef ENABLE_AMQ_MOUNT
     case 'M':
       mount_map = optarg;
       nodefault = 1;
       break;
+#endif /* ENABLE_AMQ_MOUNT */
 
     case 'P':
       amd_program_number = atoi(optarg);
@@ -393,6 +436,10 @@ main(int argc, char *argv[])
       use_udp_flag = 1;
       break;
 
+    case 'w':
+      getpwd_flag = 1;
+      break;
+
     default:
       errs = 1;
       break;
@@ -405,9 +452,9 @@ main(int argc, char *argv[])
   if (errs) {
   show_usage:
     fprintf(stderr, "\
-Usage: %s [-h host] [[-f] [-m] [-p] [-v] [-s]] | [[-u] directory ...]]\n\
+Usage: %s [-H] [-h host] [[-f] [-m] [-p] [-v] [-s]] | [[-u] directory ...]]\n\
 \t[-l logfile|\"syslog\"] [-x log_flags] [-D dbg_opts]%s\n\
-\t[-P prognum] [-T] [-U]\n",
+\t[-P prognum] [-T] [-U] [-w]\n",
 	    am_get_progname(),
 #ifdef ENABLE_AMQ_MOUNT
 	    " [-M mapent]"
@@ -419,7 +466,6 @@ Usage: %s [-h host] [[-f] [-m] [-p] [-v] [-s]] | [[-u] directory ...]]\n\
   }
 
 
-
   /* set use_udp and use_tcp flags both to on if none are defined */
   if (!use_tcp_flag && !use_udp_flag)
     use_tcp_flag = use_udp_flag = 1;
@@ -565,6 +611,34 @@ Usage: %s [-h host] [[-f] [-m] [-p] [-v] [-s]] | [[-u] directory ...]]\n\
     }
   }
 
+  /*
+   * getpwd info
+   */
+  if (getpwd_flag) {
+    char path[MAXPATHLEN+1];
+    char *wd = getcwd(path, MAXPATHLEN+1);
+    amq_mount_tree_list *mlp = amqproc_export_1((voidp) 0, clnt);
+    amq_mount_tree_p mt;
+    int i, flag;
+
+    if (!wd) {
+      perror("getcwd");
+      exit(1);
+    }
+    for (i = 0; mlp && i < mlp->amq_mount_tree_list_len; i++) {
+      mt = mlp->amq_mount_tree_list_val[i];
+      while (1) {
+	flag = 0;
+	show_pwd(mt, path, &flag);
+	if (!flag) {
+	  printf("%s\n", path);
+	  break;
+	}
+      }
+    }
+    exit(0);
+  }
+
   /*
    * Mount info
    */
@@ -585,6 +659,7 @@ Usage: %s [-h host] [[-f] [-m] [-p] [-v] [-s]] | [[-u] directory ...]]\n\
     }
   }
 
+#ifdef ENABLE_AMQ_MOUNT
   /*
    * Mount map
    */
@@ -602,6 +677,7 @@ Usage: %s [-h host] [[-f] [-m] [-p] [-v] [-s]] | [[-u] directory ...]]\n\
       perror("automount point");
     }
   }
+#endif /* ENABLE_AMQ_MOUNT */
 
   /*
    * Get Version
